#include "parseclicks.h"
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

// Function name	: QSortIntCompare
// Description	    : 
// Return type		: int 
// Argument         : const void *a
// Argument         : const void *b
static int QSortIntCompare(const void *a, const void *b)
{
	if (*(int*)a > *(int*)b)
		return 1;
	if (*(int*)a == *(int*)b)
		return 0;
	return -1;
}

// Function name	: MergeMembers
// Description	    : 
// Return type		: void 
// Argument         : int nCount
// Argument         : int *pMembers
// Argument         : int *nOutCount
// Argument         : int **ppOutMembers
static void MergeMembers(int nCount, int *pMembers, int *nOutCount, int **ppOutMembers)
{
	// make a new array big enough to hold both the output and input arrays
	int *pTemp = new int[nCount + *nOutCount];
	int *pOut = pTemp;
	// add the output members to the new array
	for (int i=0; i<*nOutCount; i++)
	{
		*pTemp = (*ppOutMembers)[i];
		pTemp++;
	}
	// add the input members to the new array
	for (i=0; i<nCount; i++)
	{
		*pTemp = pMembers[i];
		pTemp++;
	}
	// delete the original array and save the new one
	if (*ppOutMembers != NULL)
		delete *ppOutMembers;
	*ppOutMembers = pOut;
	*nOutCount = *nOutCount + nCount;
}

// Function name	: GetNumber
// Description	    : 
// Return type		: int 
// Argument         : char *&pChar
static int GetNumber(char *&pChar)
{
	char buffer[100], *pBuf = buffer;
	while (isdigit(*pChar))
	{
		*pBuf = *pChar;
		pChar++;
		pBuf++;
	}
	*pBuf = '\0';
	return atoi(buffer);
}

// Function name	: AddRange
// Description	    : 
// Return type		: void 
// Argument         : int nFirst
// Argument         : int nLast
// Argument         : int *count
// Argument         : int **members
static void AddRange(int nFirst, int nLast, int *count, int **members)
{
	if (nFirst < 0 || nLast < 0 || nLast < nFirst)
		return;

	int nCount, *pMembers;
	nCount = nLast - nFirst + 1;
	pMembers = new int[nCount];
	for (int i=0; i<nCount; i++)
		pMembers[i] = i + nFirst;
	MergeMembers(nCount, pMembers, count, members);
}

// Function name	: GetClick
// Description	    : 
// Return type		: bool 
// Argument         : char *&pChar
// Argument         : int iproc
// Argument         : int *count
// Argument         : int **members
static bool GetClick(char *&pChar, int iproc, int *count, int **members)
{
	int nFirst, nLast;
	pChar++; // advance over '('
	while ( (*pChar != ')') && (*pChar != '\0') )
	{
		nFirst = GetNumber(pChar);
		switch (*pChar)
		{
		case '.':
			pChar++; // advance over '.'
			if (*pChar != '.')
				return false;
			pChar++; // advance over '.'
			nLast = GetNumber(pChar);
			AddRange(nFirst, nLast, count, members);
			if (*pChar == ',')
				pChar++;
			break;
		case ',':
			MergeMembers(1, &nFirst, count, members);
			pChar++;
			break;
		case ')':
			MergeMembers(1, &nFirst, count, members);
			break;
		default:
			return false;
		}
	}
	if (*pChar == '\0')
		return false;
	pChar++;
	return true;
}

// Function name	: InClick
// Description	    : 
// Return type		: bool 
// Argument         : int candidate
// Argument         : int nCount
// Argument         : int *pMembers
static bool InClick(int candidate, int nCount, int *pMembers)
{
	for (int i=0; i<nCount; i++)
		if (pMembers[i] == candidate)
			return true;
	return false;
}

// Function name	: ParseClicks
// Description	    : 
// Return type		: int 
// Argument         : char *pszClicks
// Argument         : int iproc
// Argument         : int nproc
// Argument         : int *count
// Argument         : int **members
int ParseClicks(char *pszClicks, int iproc, int nproc, int *count, int **members)
{
	char *pCurrent = pszClicks;
	int nCurCount, *pCurMembers;

	// Check arguments
	if (iproc > nproc || nproc < 1 || iproc < 0 || count == NULL || members == NULL || pszClicks == NULL)
		return 1;

	*count = 0;
	*members = NULL;

	if (strlen(pszClicks) < 1)
		return 0;

	if (*pszClicks == '*')
	{
		// Special click including everyone
		*count = nproc;
		*members = new int[nproc];
		for (int i=0; i<nproc; i++)
			(*members)[i] = i;
		return 0;
	}

	while (*pCurrent == '(')
	{
		nCurCount = 0;
		pCurMembers = NULL;
		if (GetClick(pCurrent, iproc, &nCurCount, &pCurMembers))
		{
			if (InClick(iproc, nCurCount, pCurMembers))
				MergeMembers(nCurCount, pCurMembers, count, members);
			delete pCurMembers;
		}
		else
			return 1;
	}

	if (*count > 0)
	{
		// sort the resulting array
		qsort(*members, *count, 4, QSortIntCompare);
		// delete the duplicates
		int index = 0;
		for (int i=0; i<*count-1; i++)
		{
			if ((*members)[i] != (*members)[i+1])
			{
				(*members)[index] = (*members)[i];
				index++;
			}
		}
		(*members)[index] = (*members)[*count-1];
		*count = index+1;
	}
	return 0;
}
